@@ -30,7 +30,7 @@ pub use self::ddl::{
 pub use self::operator::{BinaryOperator, UnaryOperator};
 pub use self::query::{
     Cte, Fetch, Join, JoinConstraint, JoinOperator, Offset, OffsetRows, OrderByExpr, Query, Select,
-    SelectItem, SetExpr, SetOperator, TableAlias, TableFactor, TableWithJoins, Top, Values,
+    SelectItem, SetExpr, SetOperator, TableAlias, TableFactor, TableWithJoins, Top, Values, With,
 };
 pub use self::value::{DateTimeField, Value};
 
@@ -282,7 +282,13 @@ impl fmt::Display for Expr {
                 high
             ),
             Expr::BinaryOp { left, op, right } => write!(f, "{} {} {}", left, op, right),
-            Expr::UnaryOp { op, expr } => write!(f, "{} {}", op, expr),
+            Expr::UnaryOp { op, expr } => {
+                if op == &UnaryOperator::PGPostfixFactorial {
+                    write!(f, "{}{}", expr, op)
+                } else {
+                    write!(f, "{} {}", op, expr)
+                }
+            }
             Expr::Cast { expr, data_type } => write!(f, "CAST({} AS {})", expr, data_type),
             Expr::Extract { field, expr } => write!(f, "EXTRACT({} FROM {})", field, expr),
             Expr::Collate { expr, collation } => write!(f, "{} COLLATE {}", expr, collation),
@@ -299,7 +305,7 @@ impl fmt::Display for Expr {
                 results,
                 else_result,
             } => {
-                f.write_str("CASE")?;
+                write!(f, "CASE")?;
                 if let Some(operand) = operand {
                     write!(f, " {}", operand)?;
                 }
@@ -310,7 +316,7 @@ impl fmt::Display for Expr {
                 if let Some(else_result) = else_result {
                     write!(f, " ELSE {}", else_result)?;
                 }
-                f.write_str(" END")
+                write!(f, " END")
             }
             Expr::Exists(s) => write!(f, "EXISTS ({})", s),
             Expr::Subquery(s) => write!(f, "({})", s),
@@ -462,23 +468,25 @@ pub enum Statement {
     },
     /// CREATE VIEW
     CreateView {
+        or_replace: bool,
+        materialized: bool,
         /// View name
         name: ObjectName,
         columns: Vec<Ident>,
         query: Box<Query>,
-        materialized: bool,
         with_options: Vec<SqlOption>,
     },
     /// CREATE TABLE
     CreateTable {
+        or_replace: bool,
+        external: bool,
+        if_not_exists: bool,
         /// Table name
         name: ObjectName,
         /// Optional schema
         columns: Vec<ColumnDef>,
         constraints: Vec<TableConstraint>,
         with_options: Vec<SqlOption>,
-        if_not_exists: bool,
-        external: bool,
         file_format: Option<FileFormat>,
         location: Option<String>,
         query: Option<Box<Query>>,
@@ -496,7 +504,7 @@ pub enum Statement {
         /// index name
         name: ObjectName,
         table_name: ObjectName,
-        columns: Vec<Ident>,
+        columns: Vec<OrderByExpr>,
         unique: bool,
         if_not_exists: bool,
     },
@@ -550,13 +558,45 @@ pub enum Statement {
     /// `ROLLBACK [ TRANSACTION | WORK ] [ AND [ NO ] CHAIN ]`
     Rollback { chain: bool },
     /// CREATE SCHEMA
-    CreateSchema { schema_name: ObjectName },
-
-    /// ASSERT <condition> [AS <message>]
+    CreateSchema {
+        schema_name: ObjectName,
+        if_not_exists: bool,
+    },
+    /// `ASSERT <condition> [AS <message>]`
     Assert {
         condition: Expr,
         message: Option<Expr>,
     },
+    /// `DEALLOCATE [ PREPARE ] { name | ALL }`
+    ///
+    /// Note: this is a PostgreSQL-specific statement.
+    Deallocate { name: Ident, prepare: bool },
+    /// `EXECUTE name [ ( parameter [, ...] ) ]`
+    ///
+    /// Note: this is a PostgreSQL-specific statement.
+    Execute { name: Ident, parameters: Vec<Expr> },
+    /// `PREPARE name [ ( data_type [, ...] ) ] AS statement`
+    ///
+    /// Note: this is a PostgreSQL-specific statement.
+    Prepare {
+        name: Ident,
+        data_types: Vec<DataType>,
+        statement: Box<Statement>,
+    },
+    /// EXPLAIN
+    Explain {
+        /// Carry out the command and show actual run times and other statistics.
+        analyze: bool,
+        // Display additional information regarding the plan.
+        verbose: bool,
+        /// A SQL query that specifies what to explain
+        statement: Box<Statement>,
+    },
+    /// ANALYZE
+    Analyze {
+        /// Name of table
+        table_name: ObjectName,
+    },
 }
 
 impl fmt::Display for Statement {
@@ -565,6 +605,24 @@ impl fmt::Display for Statement {
     #[allow(clippy::cognitive_complexity)]
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
+            Statement::Explain {
+                verbose,
+                analyze,
+                statement,
+            } => {
+                write!(f, "EXPLAIN ")?;
+
+                if *analyze {
+                    write!(f, "ANALYZE ")?;
+                }
+
+                if *verbose {
+                    write!(f, "VERBOSE ")?;
+                }
+
+                write!(f, "{}", statement)
+            }
+            Statement::Analyze { table_name } => write!(f, "ANALYZE TABLE {}", table_name),
             Statement::Query(s) => write!(f, "{}", s),
             Statement::Insert {
                 table_name,
@@ -609,8 +667,7 @@ impl fmt::Display for Statement {
             } => {
                 write!(f, "UPDATE {}", table_name)?;
                 if !assignments.is_empty() {
-                    write!(f, " SET ")?;
-                    write!(f, "{}", display_comma_separated(assignments))?;
+                    write!(f, " SET {}", display_comma_separated(assignments))?;
                 }
                 if let Some(selection) = selection {
                     write!(f, " WHERE {}", selection)?;
@@ -629,26 +686,25 @@ impl fmt::Display for Statement {
             }
             Statement::CreateView {
                 name,
+                or_replace,
                 columns,
                 query,
                 materialized,
                 with_options,
             } => {
-                write!(f, "CREATE")?;
-                if *materialized {
-                    write!(f, " MATERIALIZED")?;
-                }
-
-                write!(f, " VIEW {}", name)?;
-
+                write!(
+                    f,
+                    "CREATE {or_replace}{materialized}VIEW {name}",
+                    or_replace = if *or_replace { "OR REPLACE " } else { "" },
+                    materialized = if *materialized { "MATERIALIZED " } else { "" },
+                    name = name
+                )?;
                 if !with_options.is_empty() {
                     write!(f, " WITH ({})", display_comma_separated(with_options))?;
                 }
-
                 if !columns.is_empty() {
                     write!(f, " ({})", display_comma_separated(columns))?;
                 }
-
                 write!(f, " AS {}", query)
             }
             Statement::CreateTable {
@@ -656,6 +712,7 @@ impl fmt::Display for Statement {
                 columns,
                 constraints,
                 with_options,
+                or_replace,
                 if_not_exists,
                 external,
                 file_format,
@@ -672,7 +729,8 @@ impl fmt::Display for Statement {
                 //   `CREATE TABLE t (a INT) AS SELECT a from t2`
                 write!(
                     f,
-                    "CREATE {external}TABLE {if_not_exists}{name}",
+                    "CREATE {or_replace}{external}TABLE {if_not_exists}{name}",
+                    or_replace = if *or_replace { "OR REPLACE " } else { "" },
                     external = if *external { "EXTERNAL " } else { "" },
                     if_not_exists = if *if_not_exists { "IF NOT EXISTS " } else { "" },
                     name = name,
@@ -691,7 +749,6 @@ impl fmt::Display for Statement {
                 if *without_rowid {
                     write!(f, " WITHOUT ROWID")?;
                 }
-
                 if *external {
                     write!(
                         f,
@@ -732,22 +789,15 @@ impl fmt::Display for Statement {
                 columns,
                 unique,
                 if_not_exists,
-            } => {
-                write!(
-                    f,
-                    "CREATE{}INDEX{}{} ON {}({}",
-                    if *unique { " UNIQUE " } else { " " },
-                    if *if_not_exists {
-                        " IF NOT EXISTS "
-                    } else {
-                        " "
-                    },
-                    name,
-                    table_name,
-                    display_separated(columns, ",")
-                )?;
-                write!(f, ");")
-            }
+            } => write!(
+                f,
+                "CREATE {unique}INDEX {if_not_exists}{name} ON {table_name}({columns})",
+                unique = if *unique { "UNIQUE " } else { "" },
+                if_not_exists = if *if_not_exists { "IF NOT EXISTS " } else { "" },
+                name = name,
+                table_name = table_name,
+                columns = display_separated(columns, ",")
+            ),
             Statement::AlterTable { name, operation } => {
                 write!(f, "ALTER TABLE {} {}", name, operation)
             }
@@ -768,13 +818,13 @@ impl fmt::Display for Statement {
                 local,
                 variable,
                 value,
-            } => {
-                f.write_str("SET ")?;
-                if *local {
-                    f.write_str("LOCAL ")?;
-                }
-                write!(f, "{} = {}", variable, value)
-            }
+            } => write!(
+                f,
+                "SET{local} {variable} = {value}",
+                local = if *local { " LOCAL" } else { "" },
+                variable = variable,
+                value = value
+            ),
             Statement::ShowVariable { variable } => write!(f, "SHOW {}", variable),
             Statement::ShowColumns {
                 extended,
@@ -782,14 +832,13 @@ impl fmt::Display for Statement {
                 table_name,
                 filter,
             } => {
-                f.write_str("SHOW ")?;
-                if *extended {
-                    f.write_str("EXTENDED ")?;
-                }
-                if *full {
-                    f.write_str("FULL ")?;
-                }
-                write!(f, "COLUMNS FROM {}", table_name)?;
+                write!(
+                    f,
+                    "SHOW {extended}{full}COLUMNS FROM {table_name}",
+                    extended = if *extended { "EXTENDED " } else { "" },
+                    full = if *full { "FULL " } else { "" },
+                    table_name = table_name,
+                )?;
                 if let Some(filter) = filter {
                     write!(f, " {}", filter)?;
                 }
@@ -815,15 +864,46 @@ impl fmt::Display for Statement {
             Statement::Rollback { chain } => {
                 write!(f, "ROLLBACK{}", if *chain { " AND CHAIN" } else { "" },)
             }
-            Statement::CreateSchema { schema_name } => write!(f, "CREATE SCHEMA {}", schema_name),
+            Statement::CreateSchema {
+                schema_name,
+                if_not_exists,
+            } => write!(
+                f,
+                "CREATE SCHEMA {if_not_exists}{name}",
+                if_not_exists = if *if_not_exists { "IF NOT EXISTS " } else { "" },
+                name = schema_name
+            ),
             Statement::Assert { condition, message } => {
                 write!(f, "ASSERT {}", condition)?;
-
                 if let Some(m) = message {
                     write!(f, " AS {}", m)?;
                 }
                 Ok(())
             }
+            Statement::Deallocate { name, prepare } => write!(
+                f,
+                "DEALLOCATE {prepare}{name}",
+                prepare = if *prepare { "PREPARE " } else { "" },
+                name = name,
+            ),
+            Statement::Execute { name, parameters } => {
+                write!(f, "EXECUTE {}", name)?;
+                if !parameters.is_empty() {
+                    write!(f, "({})", display_comma_separated(parameters))?;
+                }
+                Ok(())
+            }
+            Statement::Prepare {
+                name,
+                data_types,
+                statement,
+            } => {
+                write!(f, "PREPARE {} ", name)?;
+                if !data_types.is_empty() {
+                    write!(f, "({}) ", display_comma_separated(data_types))?;
+                }
+                write!(f, "AS {}", statement)
+            }
         }
     }
 }
@@ -842,12 +922,28 @@ impl fmt::Display for Assignment {
     }
 }
 
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub enum FunctionArg {
+    Named { name: Ident, arg: Expr },
+    Unnamed(Expr),
+}
+
+impl fmt::Display for FunctionArg {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            FunctionArg::Named { name, arg } => write!(f, "{} => {}", name, arg),
+            FunctionArg::Unnamed(unnamed_arg) => write!(f, "{}", unnamed_arg),
+        }
+    }
+}
+
 /// A function call
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Function {
     pub name: ObjectName,
-    pub args: Vec<Expr>,
+    pub args: Vec<FunctionArg>,
     pub over: Option<WindowSpec>,
     // aggregate functions may specify eg `COUNT(DISTINCT x)`
     pub distinct: bool,
@@ -892,7 +988,7 @@ impl fmt::Display for FileFormat {
             PARQUET => "PARQUET",
             AVRO => "AVRO",
             RCFILE => "RCFILE",
-            JSONFILE => "TEXTFILE",
+            JSONFILE => "JSONFILE",
         })
     }
 }
