@@ -20,8 +20,8 @@ use serde::{Deserialize, Serialize};
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Query {
     /// WITH (common table expressions, or CTEs)
-    pub ctes: Vec<Cte>,
-    /// SELECT or UNION / EXCEPT / INTECEPT
+    pub with: Option<With>,
+    /// SELECT or UNION / EXCEPT / INTERSECT
     pub body: SetExpr,
     /// ORDER BY
     pub order_by: Vec<OrderByExpr>,
@@ -35,8 +35,8 @@ pub struct Query {
 
 impl fmt::Display for Query {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        if !self.ctes.is_empty() {
-            write!(f, "WITH {} ", display_comma_separated(&self.ctes))?;
+        if let Some(ref with) = self.with {
+            write!(f, "{} ", with)?;
         }
         write!(f, "{}", self.body)?;
         if !self.order_by.is_empty() {
@@ -157,6 +157,24 @@ impl fmt::Display for Select {
     }
 }
 
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub struct With {
+    pub recursive: bool,
+    pub cte_tables: Vec<Cte>,
+}
+
+impl fmt::Display for With {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(
+            f,
+            "WITH {}{}",
+            if self.recursive { "RECURSIVE " } else { "" },
+            display_comma_separated(&self.cte_tables)
+        )
+    }
+}
+
 /// A single CTE (used after `WITH`): `alias [(col1, col2, ...)] AS ( query )`
 /// The names in the column list before `AS`, when specified, replace the names
 /// of the columns returned by the query. The parser does not validate that the
@@ -226,7 +244,7 @@ pub enum TableFactor {
         /// Arguments of a table-valued function, as supported by Postgres
         /// and MSSQL. Note that deprecated MSSQL `FROM foo (NOLOCK)` syntax
         /// will also be parsed as `args`.
-        args: Vec<Expr>,
+        args: Vec<FunctionArg>,
         /// MSSQL-specific `WITH (...)` hints such as NOLOCK.
         with_hints: Vec<Expr>,
     },
@@ -235,11 +253,17 @@ pub enum TableFactor {
         subquery: Box<Query>,
         alias: Option<TableAlias>,
     },
+    /// `TABLE(<expr>)[ AS <alias> ]`
+    TableFunction {
+        expr: Expr,
+        alias: Option<TableAlias>,
+    },
     /// Represents a parenthesized table factor. The SQL spec only allows a
     /// join expression (`(foo <JOIN> bar [ <JOIN> baz ... ])`) to be nested,
-    /// possibly several times, but the parser also accepts the non-standard
-    /// nesting of bare tables (`table_with_joins.joins.is_empty()`), so the
-    /// name `NestedJoin` is a bit of misnomer.
+    /// possibly several times.
+    ///
+    /// The parser may also accept non-standard nesting of bare tables for some
+    /// dialects, but the information about such nesting is stripped from AST.
     NestedJoin(Box<TableWithJoins>),
 }
 
@@ -278,6 +302,13 @@ impl fmt::Display for TableFactor {
                 }
                 Ok(())
             }
+            TableFactor::TableFunction { expr, alias } => {
+                write!(f, "TABLE({})", expr)?;
+                if let Some(alias) = alias {
+                    write!(f, " AS {}", alias)?;
+                }
+                Ok(())
+            }
             TableFactor::NestedJoin(table_reference) => write!(f, "({})", table_reference),
         }
     }
