@@ -10,6 +10,12 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+/// This module contains internal utilities used for testing the library.
+/// While technically public, the library's users are not supposed to rely
+/// on this module, as it will change without notice.
+//
+// Integration tests (i.e. everything under `tests/`) import this
+// via `tests/test_utils/mod.rs`.
 use std::fmt::Debug;
 
 use super::ast::*;
@@ -53,7 +59,7 @@ impl TestedDialects {
         self.one_of_identical_results(|dialect| {
             let mut tokenizer = Tokenizer::new(dialect, sql);
             let tokens = tokenizer.tokenize().unwrap();
-            f(&mut Parser::new(tokens))
+            f(&mut Parser::new(tokens, dialect))
         })
     }
 
@@ -63,13 +69,19 @@ impl TestedDialects {
         // Parser::parse_sql(&**self.dialects.first().unwrap(), sql)
     }
 
-    /// Ensures that `sql` parses as a single statement, optionally checking
-    /// that converting AST back to string equals to `canonical` (unless an
-    /// empty canonical string is provided).
+    /// Ensures that `sql` parses as a single statement and returns it.
+    /// If non-empty `canonical` SQL representation is provided,
+    /// additionally asserts that parsing `sql` results in the same parse
+    /// tree as parsing `canonical`, and that serializing it back to string
+    /// results in the `canonical` representation.
     pub fn one_statement_parses_to(&self, sql: &str, canonical: &str) -> Statement {
         let mut statements = self.parse_sql_statements(&sql).unwrap();
         assert_eq!(statements.len(), 1);
 
+        if !canonical.is_empty() && sql != canonical {
+            assert_eq!(self.parse_sql_statements(&canonical).unwrap(), statements);
+        }
+
         let only_statement = statements.pop().unwrap();
         if !canonical.is_empty() {
             assert_eq!(canonical, only_statement.to_string())
@@ -104,7 +116,9 @@ impl TestedDialects {
     /// Ensures that `sql` parses as an expression, and is not modified
     /// after a serialization round-trip.
     pub fn verified_expr(&self, sql: &str) -> Expr {
-        let ast = self.run_parser_method(sql, Parser::parse_expr).unwrap();
+        let ast = self
+            .run_parser_method(sql, |parser| parser.parse_expr())
+            .unwrap();
         assert_eq!(sql, &ast.to_string(), "round-tripping without changes");
         ast
     }
@@ -117,6 +131,7 @@ pub fn all_dialects() -> TestedDialects {
             Box::new(PostgreSqlDialect {}),
             Box::new(MsSqlDialect {}),
             Box::new(AnsiDialect {}),
+            Box::new(SnowflakeDialect {}),
         ],
     }
 }
@@ -140,3 +155,26 @@ pub fn expr_from_projection(item: &SelectItem) -> &Expr {
 pub fn number(n: &'static str) -> Value {
     Value::Number(n.parse().unwrap())
 }
+
+pub fn table_alias(name: impl Into<String>) -> Option<TableAlias> {
+    Some(TableAlias {
+        name: Ident::new(name),
+        columns: vec![],
+    })
+}
+
+pub fn table(name: impl Into<String>) -> TableFactor {
+    TableFactor::Table {
+        name: ObjectName(vec![Ident::new(name.into())]),
+        alias: None,
+        args: vec![],
+        with_hints: vec![],
+    }
+}
+
+pub fn join(relation: TableFactor) -> Join {
+    Join {
+        relation,
+        join_operator: JoinOperator::Inner(JoinConstraint::Natural),
+    }
+}
